﻿local L = AceLibrary("AceLocale-2.2"):new("Comments")


L:RegisterTranslations("enUS", function() return {

	["Comments"] = true,
	["Description"] = "wowhead.com comments on quest details.",
	
	["Minimum width"] = true,
	["Set the minimum width of the Comments frame"] = true,
	
	["Maximum height"] = true,
	["Set the maximum height of the Comments frame"] = true,

} end)

L:RegisterTranslations("deDE", function() return {

	["Comments"] = "Kommentare",
	["Description"] = "Kommentare zu Questdetails von wowhead.com.",
	
	["Minimum width"] = "Mindestbreite",
	["Set the minimum width of the Comments frame"] = "Legt die mindestbreite des Kommentarfensters fest",
	
	["Maximum height"] = "Maximale höhe",
	["Set the maximum height of the Comments frame"] = "Legt die maximale höhe des Kommentarfensters fest",

} end)

L:RegisterTranslations("koKR", function() return {

	["Comments"] = "설명",
	["Description"] = "wowhead.com의 퀘스트 상세 설명을 표시합니다.",
	
	["Minimum width"] = "최소 가로 길이",
	["Set the minimum width of the Comments frame"] = "설명 창의 최소 가로 길이를 설정합니다.",
	
	["Maximum height"] = "최소 세로 길이",
	["Set the maximum height of the Comments frame"] = "설명 창의 최소 세로 길이를 설정합니다.",

} end)

local moduleName = "Comments"
local Comments = nQuestLog:NewModule(moduleName,"AceEvent-2.0","AceHook-2.1")

local tablet = AceLibrary("Tablet-2.0")
local quixote = AceLibrary("Quixote-1.0")
local tourist = AceLibrary("Tourist-2.0")
local abacus = AceLibrary("Abacus-2.0")


function Comments:OnModuleLoad()
	self.menuName = L["Comments"]
	self.description = L["Description"]
	
	nQuestLog:RegisterDefaults(moduleName, "profile", {
		["enabled"] = true,
		
		data = {
			fontSizePercent=1.0, detached=true, 
		},

		["minWidth"] = 300,
		["maxHeight"] = 400,
	})
end

function Comments:GetMenu(oldMenu)
	local menu = {
		type="group",
		name=self.menuName,
		order=oldMenu.order,
		desc=self.description,
		args = {
			toggleEnabled = oldMenu,
			
			minWidth = {
				order = oldMenu.order + 1,
				type = "range",
				name = L["Minimum width"],
				desc = L["Set the minimum width of the Comments frame"],
				get = function()
					return self.db.profile.minWidth
				end,
				set = function(v)
					self.db.profile.minWidth = v
					if tablet:IsRegistered('nQuestLog_Comments') then
						tablet:Close('nQuestLog_Comments')
						tablet:Unregister('nQuestLog_Comments')
					end
					self:Update()
				end,
				min=50,
				max=500,
				step=1,
			},
			maxHeight = {
				order = oldMenu.order + 2,
				type = "range",
				name = L["Maximum height"],
				desc = L["Set the maximum height of the Comments frame"],
				get = function()
					return self.db.profile.maxHeight
				end,
				set = function(v)
					self.db.profile.maxHeight = v
					if tablet:IsRegistered('nQuestLog_Comments') then
						tablet:Close('nQuestLog_Comments')
						tablet:Unregister('nQuestLog_Comments')
					end
					self:Update()
				end,
				min=50,
				max=500,
				step=1,
			},
		}
	}
	return menu
end

function Comments:ToggleEnabled(v)
	self.db.profile.enabled = v
	if (v) then
		self:SecureHook(nQuestLog, "ShowLog")
		if nQuestLog:IsModuleActive('DetailsFrame') then
			self:SecureHook(nQuestLog:GetModule('DetailsFrame'), "UpdateQuestDetail")
		end
	else
		self:Unhook(nQuestLog, "ShowLog")
		if nQuestLog:IsModuleActive('DetailsFrame') then
			self:Unhook(nQuestLog:GetModule('DetailsFrame'), "UpdateQuestDetail")
		end

	end
end

function Comments:ShowLog(object, questid)
	--self.questid = questid
	if questid and LightHeaded and LightHeaded.IterateQuestComments and nQuestLog:IsModuleEnabled("DetailsFrame") then
		self:ScheduleEvent(self.Update, 0, self) --Schedule an event because tablet bugs out otherwise.
		self.altTitle = ''
		self.altLevel = nil
	end
end

function Comments:UpdateQuestDetail(object)
	if (LightHeaded and LightHeaded.IterateQuestComments) then
		self:Update()
	end
end


function Comments:GetQuestId()
	local questid
	if nQuestLog:IsModuleEnabled("DetailsFrame") then
		questid =  nQuestLog:GetModule("DetailsFrame").questid
	else
		questid = 1
	end
	
	return questid
end

function Comments:GetMobMapQuestIdByName(questname, mmQID)
	local k,v
	for k,v in pairs(mobmap_questnames) do
		if(v==questname) and (k > tonumber(mmQID)) then
			return k
		end
	end
	return nil
end

function Comments:Update()
	if not tablet:IsRegistered('nQuestLog_Comments') then
		self.altID = nil
		self.altTitle = ''
		self.altLevel = nil

		local indentSize = 14
		local qualitycolors = {
			[0] = "|cFF9D9D9D",
			[1] = "|cFFFFFFFF",
			[2] = "|cFF1EFF00",
			[3] = "|cFF0070DD",
			[4] = "|cFFA335EE",
			[5] = "|cFFFF8000"
		}
		local function round(x)
			return ceil(x - 0.5)
		end
		local function toggleDisplay(id, idtype)
			if idtype == 'npc' then
				if self.npcid == id then self.npcid = nil else self.npcid = id end
			elseif idtype == 'quest' then
				if self.qid == id then self.qid = nil else self.qid = id end
			elseif idtype == 'object' then
				if self.objid == id then self.objid = nil else self.objid = id end
			end
			self:Update()
		end
		local function urlClick(url)
			ChatFrameEditBox:Show()
			ChatFrameEditBox:SetText(url)
			ChatFrameEditBox:SetFocus()
			ChatFrameEditBox:HighlightText()
		end
		local function addURLs(cat, linkid, linktype)
			cat:AddLine('text', '->|cFF66FF66 Wowhead URL|r <-', 'func', urlClick, 'arg1',
				'http://www.wowhead.com/?'..linktype..'='..linkid, 'indentation', indentSize*2)
			if linktype == 'npc' then
				cat:AddLine('text', '->|cFF66FF66 Thottbot URL|r <-', 'func', urlClick, 'arg1',
					'http://www.thottbot.com/c'..linkid, 'indentation', indentSize*2)
				cat:AddLine('text', '->|cFF66FF66 Allakhazam URL|r <-', 'func', urlClick, 'arg1',
					'http://wow.allakhazam.com/db/mob.html?wmob='..linkid, 'indentation', indentSize*2)
			elseif linktype == 'object' then
				cat:AddLine('text', '->|cFF66FF66 Thottbot URL|r <-', 'func', urlClick, 'arg1',
					'http://www.thottbot.com/o'..linkid, 'indentation', indentSize*2)
			elseif linktype == 'quest' then
				cat:AddLine('text', '->|cFF66FF66 Thottbot URL|r <-', 'func', urlClick, 'arg1',
					'http://www.thottbot.com/q'..linkid, 'indentation', indentSize*2)
				cat:AddLine('text', '->|cFF66FF66 Allakhazam URL|r <-', 'func', urlClick, 'arg1',
					'http://wow.allakhazam.com/db/quest.html?wquest='..linkid, 'indentation', indentSize*2)
			end
		end
		local function addNpcLocs(cat, id, name)
			id = tonumber(id)
			if LightHeaded:LoadNPCData(id) then
				for c,z,x,y in LightHeaded:IterateNPCLocs(id) do
					local zoneNames = { GetMapZones(c) }
					local zone = zoneNames[z]
					local playerZone = GetRealZoneText()
					local npcWaypointInfo = name
					if playerZone ~= zone then npcWaypointInfo = name..' ('..zone..')' end
					cat:AddLine('text', '|cFF66FF66'..zone..' ['..x..', '..y..']|r', 'func', function() LightHeaded:OnHyperlinkClick(nil,
						"lhref:zcoord:"..c..":"..z..":"..x..":"..y..":"..npcWaypointInfo); end, 'indentation', indentSize*2)
				end
			else
				addURLs(cat, id, 'npc')
			end
		end
		local function itemClick(link)
			if IsControlKeyDown() then
				DressUpItemLink(link)
			elseif IsShiftKeyDown() then
				if not ChatFrameEditBox:IsVisible() then
					ChatFrameEditBox:Show()
				end
				ChatFrameEditBox:Insert(link)
			else
				ShowUIPanel(ItemRefTooltip)
				if not ItemRefTooltip:IsVisible() then
					ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE")
				end
				ItemRefTooltip:SetHyperlink(link)
			end
		end
		local function showAltQuest(id)
			local name,lvl = LightHeaded:QIDToTitleLevel(id)
			self.altID = id
			self.altTitle = name
			self.altLevel = tonumber(lvl)
		end
		tablet:Register('nQuestLog_Comments', 'detachedData', self.db.profile.data, 'cantAttach', true, 'dontHook', true, 'hideWhenEmpty', true,
			'showTitleWhenDetached', true, 'strata', 'HIGH', 'minWidth', self.db.profile.minWidth, 'maxHeight', self.db.profile.maxHeight,
			'children', function()
				local questid = self:GetQuestId()
				if questid and LightHeaded and LightHeaded.IterateQuestComments then
					local title, level, tag = quixote:GetQuestById(questid)
					local qid,sharable,_,reqlev,stype,sname,sid,etype,ename,eid,exp,rep,series
					if self.altID ~= nil and self.altTitle ~= '' and self.altLevel ~= nil then
						for iqid,isharable,_,ireqlev,istype,isname,isid,ietype,iename,ieid,iexp,irep,iseries in
							LightHeaded:IterateQuestInfo(self.altTitle, self.altLevel) do
							if tonumber(iqid) == tonumber(self.altID) then
								qid,sharable,reqlev,stype,sname,sid,etype,ename,eid,exp,rep,series = iqid,isharable,ireqlev,istype,isname,isid,
									ietype,iename,ieid,iexp,irep,iseries
							end
						end
						title = self.altTitle
						level = self.altLevel
					else
						qid,sharable,_,reqlev,stype,sname,sid,etype,ename,eid,exp,rep,series = LightHeaded:GetQuestInfo(title, level)
					end

					tablet:SetTitle("Comments")
					local cat = tablet:AddCategory('text', 'LightHeaded Info', 'columns', 1, 'wrap', true)
					local questColor = GetDifficultyColor(level)
					cat:AddLine('text', '['..level..'] '..title, 'textR', (questColor and questColor.r) or 1,
						'textG', (questColor and questColor.g) or 1, 'textB', (questColor and questColor.b) or 1, 'indentation', indentSize)
					local qinfo = 'Quest ID |cFF66FF66['..qid..']|r'
					if reqlev ~= '' then qinfo = qinfo..' - Req Lvl |cFF66FF66'..reqlev..'|r' end
					if sharable ~= '' then qinfo = qinfo..' - |cFF66FF66Sharable' end
					cat:AddLine('text', qinfo, 'func', toggleDisplay, 'arg1', qid, 'arg2', 'quest', 'indentation', indentSize)
					if qid == self.qid then addURLs(cat, qid, 'quest') end
					if sname ~= '' then
						cat:AddLine('text', 'Starts: |cFF66FF66['..sname..']', 'func', toggleDisplay, 'arg1', sid, 'arg2', stype, 'indentation',
							indentSize)
						if sid == self.npcid then addNpcLocs(cat, sid, sname) end
						if sid == self.objid then addURLs(cat, sid, stype) end
					end
					if ename ~= '' then
						cat:AddLine('text', 'Ends: |cFF66FF66['..ename..']', 'func', toggleDisplay, 'arg1', eid, 'arg2', etype, 'indentation',
							indentSize)
						if eid == self.npcid then addNpcLocs(cat, eid, ename) end
						if eid == self.objid then addURLs(cat, eid, etype) end
					end
					if exp ~= '' or rep ~= '' then
						cat = tablet:AddCategory('text', 'LightHeaded Rewards', 'columns', 1)
						if exp ~= '' then
							local effExp = exp
							local playerLevel = UnitLevel('player')
							--Quest XP formula from http://www.wowwiki.com/Formulas:Quest_XP
							--Could not determine the exact formula, but this one works well enough
							--Estimates are accurate within 3% based on limited testing
							if playerLevel == (level + 6) then
								effExp = round(exp * 0.8 / 5) * 5
							elseif playerLevel == (level + 7) then
								effExp = round(exp * 0.6 / 5) * 5
							elseif playerLevel == (level + 8) then
								effExp = round(exp * 0.4 / 5) * 5
							elseif playerLevel == (level + 9) then
								effExp = round(exp * 0.2 / 5) * 5
							elseif playerLevel >= (level + 10) then
								effExp = round(exp * 0.1 / 5) * 5
							end
							local xpLine = 'XP: |cFF66FF66'..exp
							if effExp ~= exp then xpLine = xpLine..'|r (|cFF66FF66'..effExp..'|r @ level '..playerLevel..')' end
							cat:AddLine('text', xpLine, 'indentation', indentSize)
						end
						if rep ~= '' then
							local factionTable = {}
							local factionIndex
							for factionIndex = 1, GetNumFactions() do
								local name, _, standingId, _, _, _, _, _, _, _, _ = GetFactionInfo(factionIndex)
								table.insert(factionTable, {name, standingId})
							end
							for name,value in rep:gmatch("([^\029]+)\029([^\029]+)") do
								local k,v
								for k,v in ipairs(factionTable) do
									if name == v[1] then
										local r,g,b = FACTION_BAR_COLORS[v[2]].r, FACTION_BAR_COLORS[v[2]].g, FACTION_BAR_COLORS[v[2]].b
										local hexcolor = string.format("%02x%02x%02x", r*255, g*255, b*255)
										cat:AddLine('text', 'Rep: |cFF'..hexcolor..name..' |cFF66FF66+'..value, 'indentation', indentSize)
									end
								end
							end
						end
					end --if exp ~= '' or rep ~= ''
					if MobMapDotParentFrame then
						MobMap_LoadDatabase(MOBMAP_QUEST_DATABASE)
						local mmQID = self:GetMobMapQuestIdByName(title, 0)
						local mmLevel, mmZoneID, mmIsHorde, mmIsAlliance, mmMoney, mmNPC, mmSourcePointer, mmGroupType, mmAlways, mmChoice
						if mmQID and type(MobMap_GetDetailsForQuest) == "function" then
							mmLevel, mmZoneID, _, _, mmIsHorde, mmIsAlliance, mmMoney, mmNPC, mmSourcePointer,
								mmGroupType, mmAlways, mmChoice = MobMap_GetDetailsForQuest(mmQID)
							if sname ~= '' and type(MobMap_GetMobName) == "function" and type(MobMap_GetQuestSourceName) == "function" then
								while not ((tonumber(mmLevel) == tonumber(level)) and ((sname == MobMap_GetMobName(mmNPC)) or
									(sname == MobMap_GetQuestSourceName(mmSourcePointer)))) do
									mmQID = self:GetMobMapQuestIdByName(title, mmQID)
									if mmQID == nil then break end
									mmLevel, mmZoneID, _, _, mmIsHorde, mmIsAlliance, mmMoney, mmNPC, mmSourcePointer,
										mmGroupType, mmAlways, mmChoice = MobMap_GetDetailsForQuest(mmQID)
								end
							else
								while tonumber(mmLevel) ~= tonumber(level) do
									mmQID = self:GetMobMapQuestIdByName(title, mmQID)
									if mmQID == nil then break end
									mmLevel, mmZoneID, _, _, mmIsHorde, mmIsAlliance, mmMoney, mmNPC, mmSourcePointer,
										mmGroupType, mmAlways, mmChoice = MobMap_GetDetailsForQuest(mmQID)
								end
							end
						end
						if mmQID then
							cat = tablet:AddCategory('text', 'MobMap Info', 'columns', 1)
							local faction = ''
							if mmIsHorde and mmIsAlliance then
								faction = ' |cFFFFFF00Horde/Alliance'
							elseif mmIsHorde then
								faction = ' |cFF40BF40Horde-only'
							elseif mmIsAlliance then
								faction = ' |cFF40BF40Alliance-only'
							end
							cat:AddLine('text', 'Quest Type: |cFF66FF66'..MOBMAP_QUEST_TYPES[mmGroupType]..faction, 'indentation', indentSize)
							if type(MobMap_GetZoneName) == "function" then
								cat:AddLine('text', 'Zone: |cFF66FF66'..MobMap_GetZoneName(mmZoneID), 'indentation', indentSize)
							end
							if sname == '' then
								local source = ''
								if mmSourcePointer ~= 0 and type(MobMap_GetQuestSourceName) == "function" then
									source = MobMap_GetQuestSourceName(mmSourcePointer)
								elseif mmNPC ~= 0 and type(MobMap_GetMobName) == "function" then
									source = MobMap_GetMobName(mmNPC)
								end
								if source ~= '' and type(MobMap_ParseQuestObjective) == "function" then
									cat:AddLine('text', 'Source: |cFF66FF66['..source..']', 'func',
										function() MobMap_ParseQuestObjective(source) end, 'indentation', indentSize)
								end
							end
							if #(mmChoice) > 0 or #(mmAlways) > 0 or mmMoney > 0 then
								cat = tablet:AddCategory('text', 'MobMap Rewards', 'columns', 1)
								if #(mmChoice) > 0 and type(MobMap_ConstructItemString) == "function" then
									cat:AddLine('text', 'Choose from:', 'indentation', indentSize)
									local index, id
									for index, id in pairs(mmChoice) do
										local itemString = MobMap_ConstructItemString(id)
										local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount,
											itemEquipLoc, itemTexture = GetItemInfo(itemString)
										local item = (itemRarity and qualitycolors[itemRarity] or '')..'['..(itemName or L["<not yet cached>"])..']'
										cat:AddLine('text', item, 'indentation', 4, 'checked', true, 'hasCheck', true, 'checkIcon', itemTexture,
											'func', itemClick, 'arg1', itemLink, 'indentation', indentSize+2)
									end
								end
								if #(mmAlways) > 0 and type(MobMap_ConstructItemString) == "function" then
									cat:AddLine('text', 'Always receive:', 'indentation', indentSize)
									local index, id
									for index, id in pairs(mmAlways) do
										local itemString = MobMap_ConstructItemString(id)
										local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount,
											itemEquipLoc, itemTexture = GetItemInfo(itemString)
										local item = (itemRarity and qualitycolors[itemRarity] or '')..'['..(itemName or L["<not yet cached>"])..']'
										cat:AddLine('text', item, 'indentation', 4, 'checked', true, 'hasCheck', true, 'checkIcon', itemTexture,
											'func', itemClick, 'arg1', itemLink, 'indentation', indentSize+2)
									end
								end
								if mmMoney > 0 then
									local expToCopper = 0
									if exp ~= '' then
										if tonumber(exp) >= 1000 then
											expToCopper = round((exp+1)*0.01)*60
										else
											expToCopper = round((exp+1)*0.02)*30
										end
									end
									mmMoney = mmMoney - expToCopper
									local money = abacus:FormatMoneyFull(mmMoney, true)
									if expToCopper > 0 then money = money..' (+'..abacus:FormatMoneyFull(expToCopper, true)..' @ 70)' end
									cat:AddLine('text', 'Money:|cFFFFFFFF '..money, 'indentation', indentSize)
								end
							end --if #(mmChoice) > 0 or #(mmAlways) > 0 or mmMoney > 0
						end --if mmQID
					end --if MobMapDotParentFrame
					if series ~= '' then
						cat = tablet:AddCategory('text', 'LightHeaded Quest Series', 'columns', 1)

						for step,id in series:gmatch("([^\029]+)\029([^\029]+)") do
							local name, lvl, color
							local isChecked = false
							id = tonumber(id)

							if not LH_QuestData then LoadAddOn("LightHeaded_Quest_Data") end
							name,lvl = LightHeaded:QIDToTitleLevel(id)
							color = GetDifficultyColor(lvl)

							local link = "["..lvl.."] "..name
							if tonumber(qid) == id then isChecked = true end
--							if LightHeaded:KeyMatchQID(title, level, id) then isChecked = true end
							cat:AddLine('text', link, 'textR', (color and color.r) or 1, 'textG', (color and color.g) or 1, 'textB',
								(color and color.b) or 1, 'func', showAltQuest, 'arg1', id, 'hasCheck', true, 'checked', isChecked)
						end
					end
					
					cat = tablet:AddCategory('text', 'LightHeaded Comments', 'columns', 1)
					if LightHeaded:GetNumQuestComments(title, level) then
						for idx, cqid, cid, rating, indent, parent, date, poster, comment in LightHeaded:IterateQuestComments(title, level) do
							if tonumber(qid) == tonumber(cqid) then
								cat:AddLine('text', ("|cFF66FF66[%s - %s - %s]|r"):format(poster, date, rating), 'wrap', true, 'indentation',
									indent*indentSize)
								cat:AddLine('text', comment, 'wrap', true, 'indentation', indent*indentSize)
								cat:AddLine('text', ' ') -- spacer.
							end
						end
					else
						cat:AddLine('text', L["No comments found..."])
					end
					
					cat:AddLine('text', "Comments are all from Wowhead", 'hasCheck', true, 'checked', true, 'checkIcon',
						'Interface\\Addons\\LightHeaded\\images\\wh_icon', 'func', urlClick, 'arg1', 'http://www.wowhead.com')
				end --if questid and LightHeaded and LightHeaded.IterateQuestComments
			end)
		tablet:Open('nQuestLog_Comments')
	end
	tablet:Refresh('nQuestLog_Comments')
end
